//********************************************************************
// Test and Demo Program for the E-LAB Boards
//     I2CkeyLed and I2CdigIO
// I2CkeyLeds has 8 keys as input and 8 Leds as output
// I2CdigIO has 8 opto couplers as input and 8 relais as output
// The E-LAB EvaBoard-I is used as the controlling device
//********************************************************************

program KeyLeds;

{$NOSHADOW}

Device = mega128;

Import SysTick, BeepPort, TWImaster;

From system import longword;
Define
        ProcClock   = 16000000;        {Hertz}
        SysTick     = 10;             {msec}
        StackSize   = $0058, iData;
        FrameSize   = $0100, iData;
        TWIpresc    = TWI_BR400;
        BeepPort    = PortG, 4;


Implementation

{$IDATA}
{--------------------------------------------------------------}
{ Type Declarations }
type
  tLeds     = (Led0, Led1, Led2, Led3, Led4, Led5, Led6, Led7);
  tLedSet   = bitset of tLeds;

  tKeys     = (Key0, Key1, Key2, Key3, Key4, Key5, Key6, Key7);
  tKeySet   = bitset of tKeys;

  tRelays   = (RS0, RS1, RS2, RS3, RS4, RS5, RS6, RS7);
  tRsSet    = bitset of tRelays;

  tOCinp    = (OC0, OC1, OC2, OC3, OC4, OC5, OC6, OC7);
  tOcSet    = bitset of tOCinp;

{--------------------------------------------------------------}
{ Const Declarations }
const
  TWIkeyLed      : byte = $26;
  TWIkey_RD      : byte = 0;
  TWIkey_WR      : byte = 2;
  TWIkey_DDR     : byte = 6;
  TWIkey_Pol     : byte = 4;          // polarity setting for input register only
  TWIled_RD      : byte = 1;
  TWIled_WR      : byte = 3;
  TWIled_DDR     : byte = 7;
  TWIled_Pol     : byte = 5;          // polarity setting for input register only

  TWIdigIO       : byte = $25;
  TWIdigInp_RD   : byte = 0;
  TWIdigInp_WR   : byte = 2;
  TWIdigInp_DDR  : byte = 6;
  TWIdigInp_Pol  : byte = 4;          // polarity setting for input register only
  TWIdigOut_RD   : byte = 1;
  TWIdigOut_WR   : byte = 3;
  TWIdigOut_DDR  : byte = 7;
  TWIdigOut_Pol  : byte = 5;          // polarity setting for input register only

{--------------------------------------------------------------}
{ Var Declarations }
{$IDATA}
VAR
  idx, i   : byte;
  led      : tLeds;
  KeysNew,
  KeysOld  : tKeySet;
  OCinNew,
  OCinOld  : tOCset;

{--------------------------------------------------------------}
{ functions }
procedure PortInit;
begin
  TWIout(TWIkeyLed, TWIled_DDR, $00);        // all output
  TWIout(TWIkeyLed, TWIkey_Pol, $FF);        // all keys inverted

  TWIout(TWIdigIO, TWIdigOut_DDR, $00);      // all output
  TWIout(TWIdigIO, TWIdigInp_Pol, $FF);      // all keys inverted
end;

// I2C KeyLeds card
procedure SetLedAll(leds : tLedSet);
begin
  TWIout(TWIkeyLed, TWIled_WR, (not leds));
end;

procedure SetLedOn(led : tLeds);
var
  ld             : tLedSet;
begin
  TWIout(TWIkeyLed, TWIled_RD);
  TWIinp(TWIkeyLed, ld);
  excl(ld, [led]);
  TWIout(TWIkeyLed, TWIled_WR, ld);
end;

procedure SetLedOff(led : tLeds);
var
  ld             : tLedSet;
begin
  TWIout(TWIkeyLed, TWIled_RD);
  TWIinp(TWIkeyLed, ld);
  incl(ld, [led]);
  TWIout(TWIkeyLed, TWIled_WR, ld);
end;

function KeyPressed : boolean;
begin
  KeysOld:= KeysNew;
  TWIout(TWIkeyLed, TWIkey_RD);
  TWIinp(TWIkeyLed, KeysNew);
  // a little bit debouncing
  if KeysNew <> KeysOld then
    // is bouncing
    return(false);
  else
    return(KeysNew <> []);
  endif;
end;

function GetKeySet : tKeySet;
begin
  KeysOld:= KeysNew;
  TWIout(TWIkeyLed, TWIkey_RD);
  TWIinp(TWIkeyLed, KeysNew);
  // a little bit debouncing
  if KeysNew <> KeysOld then
    // is bouncing
    return([]);
  else
    return(KeysNew);
  endif;
end;

function GetKey : tKeys;
var
  keys : tKeySet;
begin
  keys:= GetKeySet;
  case keys of
    [key0]  : return(key0);
            |
    [key1]  : return(key1);
            |
    [key2]  : return(key2);
            |
    [key3]  : return(key3);
            |
    [key4]  : return(key4);
            |
    [key5]  : return(key5);
            |
    [key6]  : return(key6);
            |
    [key7]  : return(key7);
            |
  else
    return(tKeys($ff));              // key not found, none active, more than one pressed
  endcase;
end;

// I2C DigitalIO card
procedure SetRsAll(Rel : tRsSet);
begin
  TWIout(TWIdigIO, TWIdigOut_WR, Rel);
end;

procedure SetRelOn(Relay : tRelays);
var
  rs             : tRsSet;
begin
  TWIout(TWIdigIO, TWIdigOut_RD);
  TWIinp(TWIdigIO, rs);
  incl(rs, [Relay]);
  TWIout(TWIdigIO, TWIdigOut_WR, rs);
end;

procedure SetRelOff(Relay : tRelays);
var
  rs             : tRsSet;
begin
  TWIout(TWIdigIO, TWIdigOut_RD);
  TWIinp(TWIdigIO, rs);
  excl(rs, [Relay]);
  TWIout(TWIdigIO, TWIdigOut_WR, rs);
end;

function OCinpActive : boolean;
begin
  OCinOld:= OCinNew;
  TWIout(TWIdigIO, TWIdigInp_RD);
  TWIinp(TWIdigIO, OCinNew);
  // a little bit debouncing
  if OCinNew <> OCinOld then
    // is bouncing
    return(false);
  else
    return(OCinNew <> []);
  endif;
end;

function GetOCset : tOcSet;
begin
  OCinOld:= OCinNew;
  TWIout(TWIdigIO, TWIdigInp_RD);
  TWIinp(TWIdigIO, OCinNew);
  // a little bit debouncing
  if OCinNew <> OCinOld then
    // is bouncing
    return([]);
  else
    return(OCinNew);
  endif;
end;

function GetOCinp : tOCinp;
var
  OCinp : tOcSet;
begin
  OCinp:= GetOCset;
  case OCinp of
    [OC0]  : return(OC0);
            |
    [OC1]  : return(OC1);
            |
    [OC2]  : return(OC2);
            |
    [OC3]  : return(OC3);
            |
    [OC4]  : return(OC4);
            |
    [OC5]  : return(OC5);
            |
    [OC6]  : return(OC6);
            |
    [OC7]  : return(OC7);
            |
  else
    return(tOCinp($ff));              // input not found, none active or more than one active
  endcase;
end;



{--------------------------------------------------------------}
{ Main Program }
{$IDATA}
begin
  EnableInts;
  BeepOut(2000, 5);
  mDelay(1000);
  PortInit;

  // I2C KeyLeds card
  if TWIstat(TWIkeyLed) then
    // a little led testing
    BeepOutLH;
    mDelay(200);
    BeepOutHL;
    for idx:= 0 to 7 do
      SetLedOn(tLeds(idx));
      mDelay(200);
    endfor;
    for idx:= 7 downto 0 do
      SetLedOff(tLeds(idx));
      mDelay(200);
    endfor;
    i:= 1;
    for idx:= 0 to 7 do
      SetLedAll(tLedSet(i));
      i:= i shl 1;
      mDelay(200);
    endfor;
    i:= $80;
    for idx:= 0 to 7 do
      SetLedAll(tLedSet(i));
      i:= i shr 1;
      mDelay(200);
    endfor;
    SetLedAll([]);
  else
    BeepSiren(0, 3);
  endif;

  // I2C DigitalIO card
  if TWIstat(TWIdigIO) then
    // a little led testing
    BeepOutLH;
    mDelay(200);
    BeepOutHL;
    mDelay(300);
    SetRsAll([]);
    mDelay(300);
    for idx:= 0 to 7 do
      SetRelOn(tRelays(idx));
      mDelay(200);
    endfor;
    for idx:= 7 downto 0 do
      SetRelOff(tRelays(idx));
      mDelay(200);
    endfor;
    i:= 1;
    for idx:= 0 to 7 do
      SetRsAll(tRsSet(i));
      i:= i shl 1;
      mDelay(200);
    endfor;
    i:= $80;
    for idx:= 0 to 7 do
      SetRsAll(tRsSet(i));
      i:= i shr 1;
      mDelay(200);
    endfor;
    SetRsAll([]);
  else
    BeepSiren(0, 3);
  endif;

  BeepOut(2000, 5);
  
  loop
    if KeyPressed then
      BeepClick;
      Led:= tLeds(GetKey);
      SetLedOn(led);
      SetRelOn(tRelays(led));
      mDelay(150);
      SetLedOff(led);
      SetRelOff(tRelays(led));
      mDelay(150);
    elsif OCinpActive then
      BeepClick;
      Led:= tLeds(GetOCinp);
      SetLedOn(led);
      SetRelOn(tRelays(led));
      mDelay(150);
      SetLedOff(led);
      SetRelOff(tRelays(led));
      mDelay(150);
    else
      SetLedAll([]);
      SetRsAll([]);
    endif;
 endloop;
end KeyLeds.

